
# q-expansion of newform 1176.2.q.l, downloaded from the LMFDB on 15 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1176
weight = 2
poly_data = [25, -5, -4, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[-25, -4, 4, 1], 20], [[5, 9, 1, -1], 5], [[-25, 8, 2, 3], 10]]

hecke_ring_character_values = [[295, [1, 0, 0, 0]], [589, [1, 0, 0, 0]], [785, [1, 0, 0, 0]], [1081, [-1, -1, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 1, 0, 0], [-1, -1, 0, 1], [0, 0, 0, 0], [0, -1, -1, 1], [-2, 0, -1, 0], [0, 4, 0, 0], [-2, -2, 0, -1], [-4, -4, 0, 0], [1, 0, 1, 0], [0, 1, 0, 0], [-2, -2, 0, 1], [-4, 0, 2, 0], [-4, 0, 1, 0], [-6, -6, 0, 0], [0, 5, -1, 1], [0, -3, -1, 1], [10, 10, 0, 0], [0, 4, 1, -1], [2, 0, 0, 0], [0, 0, -1, 1], [-3, -3, 0, -2], [-3, 0, -1, 0], [-2, -2, 0, -2], [-13, 0, 1, 0], [0, 6, 0, 0], [-2, -2, 0, 3], [3, 3, 0, 3], [0, 4, -3, 3], [4, 0, -2, 0], [-15, 0, 2, 0], [3, 3, 0, 1], [0, 10, 2, -2], [-4, 0, -1, 0], [6, 6, 0, 2], [0, 1, -5, 5], [0, -8, -2, 2], [12, 12, 0, 0], [6, 0, 0, 0], [-14, -14, 0, -2], [0, -8, 2, -2], [6, 0, 3, 0], [4, 4, 0, -2], [0, 9, 2, -2], [-10, 0, 2, 0], [0, -12, -4, 4], [-2, 0, -6, 0], [-3, 0, 1, 0], [0, 7, -1, 1], [14, 14, 0, 1], [-8, -8, 0, 2], [24, 0, -2, 0], [0, 3, 3, -3], [-3, 0, 5, 0], [-6, -6, 0, 4], [0, -18, -2, 2], [0, -25, -1, 1], [-3, -3, 0, 1], [0, -14, 3, -3], [-18, 0, -2, 0], [0, -4, -7, 7], [15, 0, 3, 0], [10, 0, -5, 0], [0, -4, -2, 2], [-11, -11, 0, -2], [15, 15, 0, 3], [-14, -14, 0, 3], [-7, 0, -4, 0], [0, -4, -8, 8], [6, 0, 0, 0], [0, 12, -2, 2], [-6, -6, 0, 6], [0, -5, 4, -4], [2, 2, 0, 1], [26, 0, -3, 0], [2, 2, 0, 2], [0, -2, 0, 0], [18, 18, 0, -5], [-24, -24, 0, 0], [0, -19, -2, 2], [-20, 0, -2, 0], [-4, 0, 3, 0], [0, 8, 6, -6], [-16, 0, 3, 0], [13, 13, 0, 1], [-15, -15, 0, 5], [12, 0, 2, 0], [-11, -11, 0, -6], [-22, 0, 6, 0], [26, 0, -3, 0], [32, 32, 0, 2], [0, 14, 2, -2], [0, 1, 0, 0], [7, 0, 7, 0], [-28, -28, 0, -1], [12, 0, 6, 0], [-7, -7, 0, -1], [0, 6, -2, 2], [-22, -22, 0, 1], [4, 4, 0, 1], [-28, 0, 0, 0], [0, -17, 3, -3], [0, -11, 1, -1], [-20, -20, 0, 2], [0, 4, -1, 1], [0, 25, 0, 0], [-5, 0, -1, 0], [8, 8, 0, -2], [0, 14, -2, 2], [13, 0, -4, 0], [-7, -7, 0, 0], [0, 0, 2, -2], [-6, 0, 0, 0], [0, -16, 3, -3], [7, 0, 3, 0], [0, 22, 6, -6], [16, 0, -5, 0], [0, 34, 0, 0], [-23, -23, 0, 7], [-8, 0, -8, 0], [0, -20, -3, 3], [33, 0, 2, 0], [-9, -9, 0, -1], [0, 25, 3, -3], [2, 2, 0, -3], [17, 0, -7, 0], [-6, -6, 0, 8], [-6, -6, 0, 8], [-23, 0, -2, 0], [-24, -24, 0, 7], [0, -20, -5, 5], [-10, 0, -8, 0], [19, 19, 0, -2], [-22, 0, 0, 0], [-8, -8, 0, -4], [7, 0, 6, 0], [0, 14, 4, -4], [0, 6, 2, -2], [-7, 0, 9, 0], [0, -4, 6, -6], [34, 0, 2, 0], [-17, -17, 0, 1], [0, 0, 8, -8], [3, 0, -11, 0], [0, -26, 1, -1], [-4, 0, 8, 0], [8, 0, -5, 0], [0, 22, 2, -2], [-30, -30, 0, -6], [-2, -2, 0, -4], [-4, -4, 0, -10], [-18, 0, 2, 0], [-14, 0, 5, 0], [8, 8, 0, 6], [0, -34, -1, 1], [-6, 0, 14, 0], [-10, -10, 0, 3], [-6, -6, 0, -8], [-11, 0, 4, 0], [0, -39, 3, -3], [28, 28, 0, -2], [30, 0, 6, 0], [-45, 0, -2, 0], [1, 1, 0, 13], [0, 26, 4, -4], [0, 12, 12, -12], [0, 3, 4, -4], [0, 32, -3, 3], [19, 0, -10, 0], [7, 7, 0, 3], [0, 37, -1, 1], [14, 0, -4, 0], [32, 32, 0, 4], [0, 13, 0, 0], [0, 3, 10, -10], [44, 0, 4, 0], [20, 0, -9, 0], [0, -21, -9, 9], [0, 0, -8, 0], [24, 24, 0, 1], [-23, -23, 0, 12], [3, 0, -1, 0], [12, 0, 3, 0], [46, 46, 0, -2], [0, -34, -8, 8], [0, 5, -5, 5], [0, -14, -12, 12], [0, -24, -9, 9], [9, 9, 0, -14], [0, -52, -2, 2], [-15, -15, 0, 8], [-23, 0, -5, 0], [32, 32, 0, 0], [2, 2, 0, 10], [0, -41, 5, -5], [0, 14, 2, -2], [0, 32, -1, 1], [-2, -2, 0, -4], [-14, 0, -12, 0], [22, 22, 0, -2], [0, -1, 3, -3], [6, 0, -13, 0], [38, 38, 0, 0], [0, -18, 8, -8], [-20, 0, -2, 0], [0, 11, 7, -7], [0, 0, 0, -8], [47, 47, 0, 5], [36, 0, -8, 0], [0, 38, 6, -6], [4, 4, 0, 7], [9, 0, -3, 0], [1, 0, -15, 0], [-35, -35, 0, -7], [0, -8, 0, 0], [12, 12, 0, 1], [0, 11, 9, -9], [0, 12, 8, -8], [-2, -2, 0, -2], [26, 0, -4, 0], [-34, -34, 0, -5], [-22, 0, -7, 0], [14, 14, 0, -4], [-13, -13, 0, 7], [-29, 0, 13, 0], [-20, 0, 2, 0], [-14, -14, 0, 10], [0, 8, -2, 2], [31, 31, 0, -2], [-32, -32, 0, 2], [0, 28, -7, 7], [0, -16, -12, 12], [19, 0, 2, 0], [0, -30, 4, -4], [0, 0, -9, 0], [0, 44, 2, -2], [-17, -17, 0, 2], [-11, -11, 0, -1], [20, 0, 0, 0], [0, 0, -14, 0], [0, 24, 6, -6], [22, 22, 0, -1], [0, -9, 14, -14], [-2, -2, 0, -4], [16, 0, -8, 0], [-44, -44, 0, -4], [-35, 0, 2, 0], [0, -8, 14, -14], [0, 64, 0, 0], [16, 0, 8, 0], [-48, 0, 3, 0], [22, 22, 0, 10], [0, -14, 4, -4], [8, 0, -15, 0], [0, -5, -7, 7], [-11, -11, 0, 1], [0, 60, 5, -5], [0, 2, -3, 3], [-41, 0, 1, 0], [-1, -1, 0, 13], [0, -22, -1, 1], [-41, 0, -5, 0], [0, -36, -1, 1], [-26, 0, 1, 0], [0, -18, -4, 4], [-44, -44, 0, 5], [43, 0, 5, 0], [-12, 0, 2, 0], [20, 0, 0, 0], [0, -5, 7, -7], [-50, -50, 0, 3], [0, 26, -6, 6], [0, 35, 0, 0], [-41, -41, 0, 8], [9, 0, -17, 0], [48, 48, 0, 4], [7, 7, 0, -11], [0, 2, -3, 3], [32, 32, 0, -9], [8, 8, 0, 4], [0, 2, -18, 18], [0, 23, 2, -2], [28, 0, 2, 0], [-10, 0, 13, 0], [46, 46, 0, 2], [-8, -8, 0, 2], [0, 19, -2, 2], [-29, 0, 13, 0], [0, -13, 2, -2], [56, 0, -2, 0], [0, 18, -14, 14], [0, 35, -11, 11], [0, 0, 0, -14], [-28, 0, 8, 0], [14, 0, -13, 0], [0, 5, -15, 15], [14, 14, 0, 3], [6, 0, 6, 0], [-8, -8, 0, -14], [-68, 0, -3, 0], [22, 22, 0, -16], [21, 21, 0, 11], [0, 31, 1, -1]]
